const Product = require("../models/Product.model.js");
const generalHelper = require("../helpers/GeneralHelper.js");
const uuid4 = require("uuid4");

//Get products by category
exports.getByCategory = (request, response) => {
    if(!request.query || generalHelper.isEmptyObject(request.query)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const category_id = generalHelper.mysql_real_escape_string(request.query.category_id);

    Product.getByCategory(category_id, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}

//Get product variants
exports.getVariants = (request, response) => {
    if(!request.params || generalHelper.isEmptyObject(request.params)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const internal_id = generalHelper.mysql_real_escape_string(request.params.internal_id);

    Product.getVariants(internal_id, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}

//Get product variant images
exports.getVariantImages = (request, response) => {
    if(!request.params || generalHelper.isEmptyObject(request.params)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const product_id = generalHelper.mysql_real_escape_string(request.params.product_id);

    Product.getVariantImages(product_id, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}

//Create product review
exports.addReview = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const id = uuid4();
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const product_id = generalHelper.mysql_real_escape_string(request.body.product_id);
    const internal_id = generalHelper.mysql_real_escape_string(request.body.internal_id);
    const rating = generalHelper.mysql_real_escape_string(request.body.rating);
    const heading = generalHelper.mysql_real_escape_string(request.body.heading);
    const written_review = generalHelper.mysql_real_escape_string(request.body.written_review);

    let attachments = [];
    if(request.files) {
        attachments = request.files;
    }

    if(request.user.id === user_id) {
        Product.addReview(id, user_id, product_id, internal_id, rating, heading, written_review, attachments, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        })
    }
    else {
        response.status(403).send({"code": "Unauthorized", "description": "You are not authorized to perform this action"});
    }
}

//Get reviews for a product
exports.getReviews = (request, response) => {
    if(!request.params || generalHelper.isEmptyObject(request.params)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const internal_id = generalHelper.mysql_real_escape_string(request.params.internal_id);

    Product.getReviews(internal_id, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}

//Get reviews metrics for a product
exports.getReviewMetrics = (request, response) => {
    if(!request.params || generalHelper.isEmptyObject(request.params)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const internal_id = generalHelper.mysql_real_escape_string(request.params.internal_id);

    Product.getReviewMetrics(internal_id, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}