const User = require("../models/User.model.js");
const generalHelper = require("../helpers/GeneralHelper.js");
const uuid4 = require("uuid4");

//Refresh access tokens
exports.refreshAccessTokens = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const refreshAccessToken = generalHelper.mysql_real_escape_string(request.body.refreshAccessToken);

    User.refreshAccessTokens(refreshAccessToken, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}
//Register User
exports.register = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    var id = uuid4();
    var number_email = generalHelper.mysql_real_escape_string(request.body.number_email);
    var name = generalHelper.mysql_real_escape_string(request.body.name);
    var mobile = generalHelper.mysql_real_escape_string(request.body.mobile);
    var image = generalHelper.mysql_real_escape_string(request.body.image);
    var type = generalHelper.mysql_real_escape_string(request.body.type);

    User.register(id, number_email, name, mobile, image, type, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}
//Login User
exports.login = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const number_email = generalHelper.mysql_real_escape_string(request.body.number_email);
    const type = generalHelper.mysql_real_escape_string(request.body.type);
    User.login(number_email, type, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}
//Check  User Email
exports.checkMail = (request, response) => {
    const number_email = generalHelper.mysql_real_escape_string(request.query.email);
    User.checkMail(number_email, (err, data) => {
        if (err) {
            if (err.code === "SUSPENDED")
              response.status(402).send({
                error: err,
            });
        } else {
        response.status(200).send(data);
        }
    });
}
// Check Email
exports.CheckMobile = (request, response) => {
    const mobile = generalHelper.mysql_real_escape_string(request.query.mobile);
    User.CheckMobile(mobile,(err, data) => {
        if (err){
         if(err.code === 'SUSPENDED')
            response.status(402).send({error: err
            });  
        }else{
            response.status(200).send(data);
        } 
    });
};
//Logout User
exports.logout = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const refreshAccessToken = generalHelper.mysql_real_escape_string(request.body.refreshAccessToken);

    User.logout(refreshAccessToken, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}
//Get addresses
exports.getAddresses = (request, response) => {
    if(!request.params || generalHelper.isEmptyObject(request.params)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const user_id = generalHelper.mysql_real_escape_string(request.params.user_id);

    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
        User.getAddresses(user_id, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
//Get addresses
exports.getGuestAddresses = (request, response) => {
    User.getGuestAddresses((err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}
//Add address
exports.addAddress = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const id = uuid4();
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const full_name = generalHelper.mysql_real_escape_string(request.body.full_name);
    const mobile_number = generalHelper.mysql_real_escape_string(request.body.mobile_number);
    const pin_code = generalHelper.mysql_real_escape_string(request.body.pin_code);
    const house_number = generalHelper.mysql_real_escape_string(request.body.house_number);
    const street = generalHelper.mysql_real_escape_string(request.body.street);
    const landmark = generalHelper.mysql_real_escape_string(request.body.landmark);
    const town_city = generalHelper.mysql_real_escape_string(request.body.town_city);
    const state = generalHelper.mysql_real_escape_string(request.body.state);

    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
        User.addAddress(id, user_id, full_name, mobile_number, pin_code, house_number, street, landmark, town_city, state, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
//Add guest address
exports.addGuestAddress = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const id = uuid4();
    // const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const full_name = generalHelper.mysql_real_escape_string(request.body.full_name);
    const mobile_number = generalHelper.mysql_real_escape_string(request.body.mobile_number);
    const pin_code = generalHelper.mysql_real_escape_string(request.body.pin_code);
    const house_number = generalHelper.mysql_real_escape_string(request.body.house_number);
    const street = generalHelper.mysql_real_escape_string(request.body.street);
    const landmark = generalHelper.mysql_real_escape_string(request.body.landmark);
    const town_city = generalHelper.mysql_real_escape_string(request.body.town_city);
    const state = generalHelper.mysql_real_escape_string(request.body.state);

    User.addGuestAddress(id, full_name, mobile_number, pin_code, house_number, street, landmark, town_city, state, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
    
}
//Get orders
exports.getOrders = (request, response) => {
    if(!request.params || generalHelper.isEmptyObject(request.params)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const user_id = generalHelper.mysql_real_escape_string(request.params.user_id);

    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
        User.getOrders(user_id, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
//Add order
exports.addOrder = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const id = uuid4();
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const address_id = generalHelper.mysql_real_escape_string(request.body.address_id);
    const razorpay_order_id = generalHelper.mysql_real_escape_string(request.body.razorpay_order_id);
    const order_amount = generalHelper.mysql_real_escape_string(request.body.order_amount);
    const order_details = generalHelper.mysql_real_escape_string(request.body.order_details);
    const product_size = generalHelper.mysql_real_escape_string(request.body.product_size);

    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
        // console.log(id, user_id, address_id, razorpay_order_id, order_amount, order_details, product_size);
        User.addOrder(id, user_id, address_id, razorpay_order_id, order_amount, order_details, product_size, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
//Add Guest Order
exports.addGuestOrder = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const id = uuid4();
    const email = generalHelper.mysql_real_escape_string(request.body.email);
    const username = generalHelper.mysql_real_escape_string(request.body.username);
    const address = generalHelper.mysql_real_escape_string(request.body.address);
    const razorpay_order_id = generalHelper.mysql_real_escape_string(request.body.razorpay_order_id);
    const order_amount = generalHelper.mysql_real_escape_string(request.body.order_amount);
    const order_details = generalHelper.mysql_real_escape_string(request.body.order_details);
    const product_size = generalHelper.mysql_real_escape_string(request.body.product_size);

    User.addGuestOrder(id,email, username, address, razorpay_order_id, order_amount, order_details, product_size, (err, data) => {
        if(err)
            response.status(401).send(err);
        else
            response.status(200).send(data);
    });
}
//Cancel order
exports.cancelOrder = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }

    const orderId = generalHelper.mysql_real_escape_string(request.body.orderId);
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    
    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
        User.cancelOrder(user_id, orderId, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
// Add Cart Item
exports.addCartItem = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const cart_id = generalHelper.mysql_real_escape_string(request.body.cart_id);
    const product_id = generalHelper.mysql_real_escape_string(request.body.product_id);
    const product_price = generalHelper.mysql_real_escape_string(request.body.product_price);
    const qty = generalHelper.mysql_real_escape_string(request.body.qty);
    const product_size = generalHelper.mysql_real_escape_string(request.body.product_size);
    
    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
       
        User.addCartItem(user_id, cart_id, product_id, product_price, qty, product_size, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
// Update Cart Item
exports.updateCartItem = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const cart_id = generalHelper.mysql_real_escape_string(request.body.cart_id);
    const qty = generalHelper.mysql_real_escape_string(request.body.qty);
   
    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
       
        User.updateCartItem(user_id, cart_id, qty, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
// get Cart Item
exports.getCartDetails = (request, response) => {
    const user_id = generalHelper.mysql_real_escape_string(request.query.user_id);
    // const cart_id = generalHelper.mysql_real_escape_string(request.query.cart_id);
    
    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }
    else {
       
        User.getCartDetails(user_id, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
// Delete Cart Item
exports.deleteCartItem = (request, response) => {
    if(!request.body || generalHelper.isEmptyObject(request.body)) {
        response.status(400).send({
            message: "Invalid request format!"
        });
        return;
    }
    const user_id = generalHelper.mysql_real_escape_string(request.body.user_id);
    const cart_id = generalHelper.mysql_real_escape_string(request.body.cart_id);
    const qty = generalHelper.mysql_real_escape_string(request.body.qty);
   
    if(user_id !== request.user.id) {
        response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
    }else {
        User.deleteCartItem(user_id,cart_id,qty, (err, data) => {
            if(err)
                response.status(401).send(err);
            else
                response.status(200).send(data);
        });
    }
}
// get Cart Item
// exports.getCart = (request, response) => {
//     const user_id = generalHelper.mysql_real_escape_string(request.query.user_id);
//     if(user_id !== request.user.id) {
//         response.status(403).send({"code": "Unauthorized", "message": "You are not authorized to perform this action"});
//     }
//     else {
       
//         User.getCart(user_id, (err, data) => {
//             if(err)
//                 response.status(401).send(err);
//             else
//                 response.status(200).send(data);
//         });
//     }
// }