const express = require("express");
const fileUpload = require('express-fileupload');
var https = require('http');
const fs = require('fs');
const path = require('path');
const generalHelper = require('./backend/helpers/GeneralHelper');
const Razorpay = require("razorpay");
require('dotenv').config();
require('./backend/config/db.config.js');

const app = express();
app.use(express.json({limit: "200mb"}));
app.use(express.urlencoded({extended: true}));
app.use(fileUpload());

app.use((_request, response, next) => {
    response.setHeader('Access-Control-Allow-Origin', '*');
    response.setHeader('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept, Authorization');
    response.setHeader('Access-Control-Allow-Methods', 'GET, POST, PATCH, DELETE, PUT, OPTIONS');
    next();
});

// Certificate
// const privateKey = fs.readFileSync('/etc/letsencrypt/live/smilewithpets.com/privkey.pem', 'utf8');
// const certificate = fs.readFileSync('/etc/letsencrypt/live/smilewithpets.com/cert.pem', 'utf8');
// const ca = fs.readFileSync('/etc/letsencrypt/live/smilewithpets.com/chain.pem', 'utf8');

// const credentials = {
//     key: privateKey,
//     cert: certificate,
//     ca: ca
// };

//Add User routes
require("./backend/routes/User.routes.js")(app);

//Add Admin routes
require("./backend/routes/Admin.routes.js")(app);

//Add Product routes
require("./backend/routes/Product.routes.js")(app);

//add Admin product routes
require("./backend/routes/AdminProduct.routes.js")(app);

//add Admin orders routes
require("./backend/routes/Adminorder.routes.js")(app);

//add Category routes
require("./backend/routes/Category.routes.js")(app);

//add Offers routes
require("./backend/routes/offers.routes.js")(app);

//add Intelligent routes
require("./backend/routes/intelligent.routes.js")(app);

//add Get Pet routes
require("./backend/routes/GetPet.routes.js")(app);

//add Appointment routes
require("./backend/routes/Appointment.routes.js")(app);

//add Newsletter routes
require("./backend/routes/Newsletter.routes.js")(app);

//add Brand routes
require("./backend/routes/Brand.routes.js")(app);
//add Demo Api routes
require("./backend/routes/ApiUser.routes.js")(app);

//create-order-razorpay
app.post("/razorpay/order/create", generalHelper.authenticateToken, async (request, response) => {
    try {
        const instance = new Razorpay({
            key_id: process.env.RAZORPAY_KEY_ID,
            key_secret: process.env.RAZORPAY_SECRET,
        });

        const options = {
            amount: request.body.amount,
            currency: request.body.currency,
            receipt: request.body.receipt,
        };
        const order = await instance.orders.create(options);

        if(order) {
            response.status(200).send({key_id:process.env.RAZORPAY_KEY_ID,order_details:order});
        }
        else { 
            response.status(500).send({"code": "ORDER ERROR", "message": "Something went wrong"});
        }
    }
    catch(error) {
        response.status(500).send({"code": "LIBRARY ERROR", "message": error});
    }
})
app.post("/razorpay/order/buy-now/create", generalHelper.authenticateToken, async (request, response) => {
    try {
        const instance = new Razorpay({
            key_id: process.env.RAZORPAY_KEY_ID,
            key_secret: process.env.RAZORPAY_SECRET,
        });
        var queryResult;
        conn.query(`SELECT discounted_price FROM products WHERE id='${request.body.amount}'`, async(err, res) => {
            if(err) {
                result({"code": "SERVER-ERROR", "description": err}, null);
                return;
            }
            else {
                // queryResult = res[0].discounted_price;
                const options = {
                    amount: res[0].discounted_price < 500 ? (res[0].discounted_price*100)+ (60 * 100): res[0].discounted_price,
                    currency: request.body.currency,
                    receipt: request.body.receipt,
                };
                const order = await instance.orders.create(options);

                if(order) {
                    response.status(200).send({key_id:process.env.RAZORPAY_KEY_ID,order_details:order,price:res[0].discounted_price});
                }
                else { 
                    response.status(500).send({"code": "ORDER ERROR", "message": "Something went wrong"});
                }
            }
        });
    }
    catch(error) {
        response.status(500).send({"code": "LIBRARY ERROR", "message": error});
    }
})
// create guest order in razorpay
app.post("/razorpay/guest-order/create", async (request, response) => {
    try {
        const instance = new Razorpay({
            key_id: process.env.RAZORPAY_KEY_ID,
            key_secret: process.env.RAZORPAY_SECRET,
        });

        const options = {
            amount: request.body.amount,
            currency: request.body.currency,
            receipt: request.body.receipt,
        };
        const order = await instance.orders.create(options);

        if(order) {
            response.status(200).send(order);
        }
        else { 
            response.status(500).send({"code": "ORDER ERROR", "message": "Something went wrong"});
        }
    }
    catch(error) {
        response.status(500).send({"code": "LIBRARY ERROR", "message": error});
    }
})
const httpServer = https.createServer(app);
// const httpServer = https.createServer(credentials, app);

// set port, listen for requests
httpServer.listen(process.env.NODE_PORT, () => {
    console.log('Server is running on port '+process.env.NODE_PORT);
});
